@@ -34,6 +34,7 @@ def new_tree(args) -> Tree:
         args.birth,
         args.death,
         min_leaves=args.min_leaves,
+        num_leaves=args.num_leaves,
         max_time=args.max_time,
         labels=args.labels,
         lam=args.lam,
@@ -79,6 +80,7 @@ def parse_arguments():
         "labels": "human",
         "max_time": None,
         "min_leaves": 10,
+        "num_leaves": None,
         "seed": None,
         "birth": 1.0,
         "output": "newick",
